@@ -1,3 +1,6 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
 import datetime
 
 import lauteur
@@ -24,6 +27,17 @@ except ImportError:
 else:
   imported_pageone = True
 
+
+try:
+  import gevent
+  from gevent.pool import Pool
+  import gevent.monkey
+  gevent.monkey.patch_all()
+except ImportError:
+  imported_gevent = False
+else:
+  imported_gevent = True
+
 # JSONPATH CANDIDATES
 URL_CANDIDATE_JSONPATH = [
   'id', 'feedburner_origlink', 'link', 'link[*].href'
@@ -65,7 +79,14 @@ class SupeRSS:
     # required arg
     self.feed = feed
     self.homepage = homepage
-    self.ignore_urls = kwargs.get('ignore_urls', set())
+    self.concurrent = kwargs.get('concurrent', False)
+    if self.concurrent:
+      if not imported_gevent:
+        raise SupeRSSInitError(
+          'SupeRSS requires gevent to run concurrently.'
+          )
+      else:
+        self.pool = Pool(kwargs.get('num_workers', 10))
 
     # if we haven't import particle, assume we have the full text
     if imported_particle:
@@ -80,7 +101,6 @@ class SupeRSS:
           'to parse a build a feed from a homepage.'
           )
 
-
   def get_candidates(self, obj, jsonpath):
     """
     evaluate an object with jsonpath, 
@@ -241,33 +261,43 @@ class SupeRSS:
     d['article_imgs'] = list(set(feed_data['article_imgs'] + extracted_data['article_imgs']))
 
     return d 
-
-  def run_homepage(self):
-    pg1 = PageOne(url=self.homepage)
+  
+  def extract_and_merge(self, entry):
+    """
+    runner for non-full-text
+    """
     p = Particle()
-    for a in pg1.articles():
-      yield p.extract(a)
+    extracted_data = p.extract(url=entry['url'])
+    return self.merge_extracted_data(entry, extracted_data)
 
-  def run_non_full_text(self):
-    p = Particle()
-
-    for entry in self.run_full_text():
-      extracted_data = p.extract(url=entry['url'])
-      yield self.merge_extracted_data(entry, extracted_data)
+  def run_concurrent(self, func, iterator):
+    if self.concurrent:
+      for result in self.pool.imap_unordered(func, iterator):
+        yield result 
+    else:
+      for item in iterator:
+        yield func(item)
 
   def run_full_text(self):
     """
     parse feed and stream entries
     """
     f = feedparser.parse(self.feed)
+    return self.run_concurrent(self.parse_entry, f.entries)
 
-    # go latest to earliest, so when
-    # we check for duplicates we get them first.
-    entries = reversed(f.entries)
-    for entry in f.entries:
-      yield self.parse_entry(entry)
+  def run_non_full_text(self):
+    """
+    Parse feed, extract article test, merge
+    """
+    return self.run_concurrent(self.extract_and_merge, self.run_full_text())
 
-  def _run(self):
+  def run_homepage(self):
+    pg1 = PageOne(url=self.homepage)
+    p = Particle()
+    return self.run_concurrent(p.extract, pg1.articles())
+
+
+  def run(self):
     """
     run different methods based on intialization parameters
     """
@@ -281,10 +311,3 @@ class SupeRSS:
 
       else:
         return self.run_non_full_text()
-
-  def run(self):
-    for entry in self._run():
-      if entry and entry['url'] not in self.ignore_urls:
-        yield entry
-
-
